# -*- coding: utf-8 -*-

# Standard Library
import datetime

# Vendor Library
from django.db.models import Q

# Project Library
from kinder.core.declaration_status.models import DSS
from kinder.core.privilege.models import Privilege
from kinder.core.dict.models import HealthNeed
from gisdo.algorithm import (
    BaseQueueAlgorithm, BaseHealthNeedAlgorithm,
    BaseActualCallAlgorithm, BaseDeferredQueueAlgorithm,
    BaseQueueWithDesiredDateInPeriodAlgorithm,
    BaseQueueAlgorithmWithOutWantChangeDou)
from gisdo.constants import REGIONAL, MUNICIPAL
from gisdo.utils import DateMixin


class CommonQueueAlgorithm(BaseQueueAlgorithm):
    """
    Общая очередь
    """

    def get_result_set(self, **kwargs):

        return super(CommonQueueAlgorithm, self).get_result_set(**kwargs)


class ShortStayQueueAlgorithm(BaseQueueAlgorithm, DateMixin):

    def get_result_set(self, **kwargs):

        qs = super(ShortStayQueueAlgorithm, self).get_result_set(**kwargs)
        qs = filter(
            lambda x: x['status__code'] in [
                DSS.REGISTERED, DSS.PRIV_CONFIRMATING]
            and (not x['desired_date']
                 or x['desired_date'] <= self.get_desired_date())
            and x['work_type__code'] == '3-5', qs)
        return filter(lambda x: x['work_type__code'] == '3-5', qs)


class PrivilegeQueueAlgorithm(
        BaseQueueAlgorithmWithOutWantChangeDou, DateMixin):

    def get_result_set(self, **kwargs):

        qs = super(PrivilegeQueueAlgorithm, self).get_result_set(**kwargs)

        qs = filter(lambda x: x['best_privilege__id'] and
                    x['status__code'] == DSS.REGISTERED and
                    (not x['desired_date']
                     or x['desired_date'] <= DateMixin.get_desired_date()),
                    qs)

        privilege_type = kwargs.get('privilege_type', None)
        if privilege_type:
            if privilege_type == REGIONAL:
                privilege_filter = Q(type=REGIONAL) | Q(type=MUNICIPAL)
            else:
                privilege_filter = Q(type=privilege_type)
            qs_pr = Privilege.objects.filter(
                privilege_filter).values_list('id', flat=True)
            qs = filter(lambda x: x['best_privilege__id'] in qs_pr, qs)

        return qs


class ActualCallAlgorithm(BaseActualCallAlgorithm):
    """
    Актуальный спрос <= 1.9
    """


class CurrentCalendarActualCallAlgorithm(BaseActualCallAlgorithm):
    """
    Актуальный спрос <= 31.08
    """

    LAST_ACADEMIC_DAY = 31
    LAST_ACADEMIC_MONTH = 8


class CurrentDateActualCallAlgorithm(BaseActualCallAlgorithm):
    """
    Актуальный спрос <= текущая дата
    """

    @classmethod
    def get_desired_date(cls):

        return datetime.date.today()


class DisabledHealthNeedAlgorithm(BaseHealthNeedAlgorithm):
    """
    Ограниченые потребности по здоровью
    """

    def _get_heath_needs(self):

        return HealthNeed.objects.exclude(
            name=u'Нет').values_list('id', flat=True)


class CorrectHealthNeedAlgorithm(BaseHealthNeedAlgorithm):
    """
    Очередь в компенсирующие группы
    """

    def _get_heath_needs(self):
        #id потребностей из документа расчета показателей
        return range(1, 10) + [12, 13]


class HealthNeedAlgorithm(BaseHealthNeedAlgorithm):
    """
    Очередь в оздоровительные группы
    """

    def _get_heath_needs(self):
        return [14, 15, 16]


class DeferredQueueAlgorithm(BaseDeferredQueueAlgorithm):
    """
    Отложенный спрос > 1.9
    """


class SoftDeferredQueueAlgorithm(
        BaseQueueAlgorithmWithOutWantChangeDou, DateMixin):
    """
    Базовый алгоритм для вычисления отложенного спроса
    """

    def get_result_set(self, **kwargs):

        qs = super(SoftDeferredQueueAlgorithm, self).get_result_set(
            **kwargs)
        return filter(
            lambda x: x['date']
            and x['date'] >= datetime.datetime.combine(
                self._get_current_calendar_year(),
                datetime.datetime.min.time()),
            qs)


class CurrentDateDeferredQueueAlgorithm(BaseDeferredQueueAlgorithm):
    """
    Отложенный спрос > текущая дата
    """

    @classmethod
    def get_desired_date(cls):

        return datetime.date.today()


class DeferredInFutureAlgorithm(BaseQueueAlgorithm, DateMixin):
    """
    Отложенный спрос >= 1.09.следующего учебного года
    """

    def get_result_set(self, **kwargs):

        qs = super(DeferredInFutureAlgorithm, self).get_result_set(**kwargs)
        return filter(
            lambda x: x['desired_date']
            and x['desired_date'] >= self.get_desired_date(), qs)

    @classmethod
    def get_desired_date(cls):

        return cls._get_next_calendar_year()


class QueueInCurrentCalendarAlgorithm(BaseQueueAlgorithm, DateMixin):
    """
    Дата подачи от >= 1.09.текущего учебного года
    Дата желаемого зачисления > текущая дата и < 1.09.следующего учебного года
    """

    def get_result_set(self, **kwargs):

        today = datetime.date.today()

        qs = super(
            QueueInCurrentCalendarAlgorithm, self).get_result_set(
                **kwargs)
        return filter(
            lambda x: x['desired_date']
            and today < x['desired_date'] < self._get_next_calendar_year()
            and x['date'] >= datetime.datetime.combine(
                self._get_current_calendar_year(),
                datetime.datetime.min.time()), qs)


class QueueInNextYear(BaseQueueWithDesiredDateInPeriodAlgorithm):

    @classmethod
    def get_desired_period(cls):
        """
        [2.09.текущего учебного года,
        31.08.следующего учебного года]
        """

        down = cls._get_current_calendar_year()
        down = datetime.date(down.year, down.month, 2)

        up = cls._get_next_calendar_year()
        up = datetime.date(up.year, 8, 31)

        return down, up


class QueueInThisYear(BaseQueueWithDesiredDateInPeriodAlgorithm):

    @classmethod
    def get_desired_period(cls):
        """
        [1.9.текущего учебного года, текущая дата]
        """

        down = cls._get_current_calendar_year()
        up = datetime.date.today()

        return down, up


class DeferredQueueInNextYear(BaseQueueWithDesiredDateInPeriodAlgorithm):

    @classmethod
    def get_desired_period(cls):
        """
        [2.09.следующего учебного года,
        31.08.следующего учебного года + 1]
        """

        down = cls._get_next_calendar_year()
        down = datetime.date(down.year, down.month, 2)

        up = cls._get_next_calendar_year()
        up = datetime.date(up.year + 1, 8, 31)

        return down, up
